@@ -129,8 +129,6 @@ use std::fmt;
 use std::iter::FromIterator;
 use std::mem;
 
-use memchr::memchr;
-
 pub use self::autiter::{
     Automaton, Match,
     Matches, MatchesOverlapping, StreamMatches, StreamMatchesOverlapping,
@@ -284,20 +282,8 @@ impl<P: AsRef<[u8]>, T: Transitions> Automaton<P> for AcAutomaton<P, T> {
     }
 
     #[inline]
-    fn skip_to(&self, si: StateIdx, text: &[u8], at: usize) -> usize {
-        if si != ROOT_STATE || !self.is_skippable() {
-            return at;
-        }
-        let b = self.start_bytes[0];
-        match memchr(b, &text[at..]) {
-            None => text.len(),
-            Some(i) => at + i,
-        }
-    }
-
-    #[inline]
-    fn is_skippable(&self) -> bool {
-        self.start_bytes.len() == 1
+    fn start_bytes(&self) -> &[u8] {
+        &self.start_bytes
     }
 
     #[inline]
