@@ -20,7 +20,7 @@ pub fn test_simple_head() {
     let res = res.unwrap();
 
     assert!(res.get_code() == 200, "code is {}", res.get_code());
-    assert!(res.get_body() == []);
+    assert!(res.get_body().len() == 0);
     assert!(res.get_headers().len() == 1);
     assert!(res.get_header("content-length") == ["5".to_string()]);
 }
