@@ -407,6 +407,9 @@ impl Config {
             if opt_level != 0 {
                 cmd.args.push("/O2".into());
             }
+            if target.contains("i686") {
+                cmd.args.push("/SAFESEH".into());
+            }
         } else if nvcc {
             cmd.args.push(format!("-O{}", opt_level).into());
         } else {
@@ -444,6 +447,14 @@ impl Config {
             if target == "armv7-unknown-linux-gnueabihf" {
                 cmd.args.push("-march=armv7-a".into());
             }
+            if target == "arm-unknown-linux-gnueabihf" {
+                cmd.args.push("-march=armv6".into());
+                cmd.args.push("-marm".into());
+            }
+            if target == "arm-unknown-linux-gnueabi" {
+                cmd.args.push("-march=armv6".into());
+                cmd.args.push("-marm".into());
+            }
         }
 
         if self.cpp && !msvc {
@@ -488,6 +499,14 @@ impl Config {
                 cmd.arg(&format!("/D{}", key));
             }
         }
+
+        if target.contains("i686") {
+            cmd.arg("/safeseh");
+        }
+        for flag in self.flags.iter() {
+            cmd.arg(flag);
+        }
+
         (cmd, tool.to_string())
     }
 
@@ -614,6 +633,7 @@ impl Config {
                     "powerpc64-unknown-linux-gnu" => Some("powerpc-linux-gnu"),
                     "powerpc64le-unknown-linux-gnu" => Some("powerpc64le-linux-gnu"),
                     "mips-unknown-linux-gnu" => Some("mips-linux-gnu"),
+                    "mipsel-unknown-linux-gnu" => Some("mipsel-linux-gnu"),
                     "i686-pc-windows-gnu" => Some("i686-w64-mingw32"),
                     "x86_64-pc-windows-gnu" => Some("x86_64-w64-mingw32"),
                     "x86_64-unknown-linux-musl" => Some("musl"),
