@@ -1,4 +1,5 @@
 use std::marker;
+use std::mem;
 use std::slice;
 
 use {raw, Oid, Object};
@@ -38,6 +39,14 @@ impl<'repo> Blob<'repo> {
             &*(self as *const _ as *const Object<'repo>)
         }
     }
+
+    /// Consumes Blob to be returned as an `Object`
+    pub fn into_object(self) -> Object<'repo> {
+        assert_eq!(mem::size_of_val(&self), mem::size_of::<Object>());
+        unsafe {
+            mem::transmute(self)
+        }
+    }
 }
 
 impl<'repo> Binding for Blob<'repo> {
@@ -78,6 +87,7 @@ mod tests {
         assert!(blob.is_binary());
 
         repo.find_object(id, None).unwrap().as_blob().unwrap();
+        repo.find_object(id, None).unwrap().into_blob().ok().unwrap();
     }
 
     #[test]
@@ -89,5 +99,6 @@ mod tests {
         let id = repo.blob_path(&path).unwrap();
         let blob = repo.find_blob(id).unwrap();
         assert_eq!(blob.content(), [7, 8, 9]);
+        blob.into_object();
     }
 }
