@@ -1,3 +1,4 @@
+use std::env::JoinPathsError;
 use std::ffi::{CStr, NulError};
 use std::error;
 use std::fmt;
@@ -214,6 +215,12 @@ impl From<NulError> for Error {
     }
 }
 
+impl From<JoinPathsError> for Error {
+    fn from(e: JoinPathsError) -> Error {
+        Error::from_str(error::Error::description(&e))
+    }
+}
+
 
 #[cfg(test)]
 mod tests {
