@@ -94,6 +94,7 @@ pub use diff::{DiffBinary, DiffBinaryFile, DiffBinaryKind};
 pub use merge::{AnnotatedCommit, MergeOptions};
 pub use error::Error;
 pub use index::{Index, IndexEntry, IndexEntries, IndexMatchedPath};
+pub use message::{message_prettify, DEFAULT_COMMENT_CHAR};
 pub use note::{Note, Notes};
 pub use object::Object;
 pub use oid::Oid;
@@ -277,7 +278,7 @@ pub enum ResetType {
 /// An enumeration all possible kinds objects may have.
 #[derive(PartialEq, Eq, Copy, Clone, Debug)]
 pub enum ObjectType {
-    /// An object which corresponds to a any git object
+    /// Any kind of git object
     Any,
     /// An object which corresponds to a git commit
     Commit,
@@ -410,6 +411,20 @@ Flags for APIs that add files matching pathspec
     }
 }
 
+bitflags! {
+    #[doc = "
+Flags for `Repository::open_ext`
+"]
+    flags RepositoryOpenFlags: u32 {
+        /// Only open the specified path; don't walk upward searching.
+        const REPOSITORY_OPEN_NO_SEARCH = raw::GIT_REPOSITORY_OPEN_NO_SEARCH as u32,
+        /// Search across filesystem boundaries.
+        const REPOSITORY_OPEN_CROSS_FS = raw::GIT_REPOSITORY_OPEN_CROSS_FS as u32,
+        /// Force opening as bare repository, and defer loading its config.
+        const REPOSITORY_OPEN_BARE = raw::GIT_REPOSITORY_OPEN_BARE as u32,
+    }
+}
+
 bitflags! {
     #[doc = "
 Flags for the return value of `Repository::revparse`
@@ -447,6 +462,7 @@ mod diff;
 mod merge;
 mod error;
 mod index;
+mod message;
 mod note;
 mod object;
 mod oid;
