@@ -281,6 +281,9 @@ pub const SA_NODEFER: ::c_int = 0x00000010;
 pub const SA_NOCLDWAIT: ::c_int = 0x00010000;
 pub const SA_NOCLDSTOP: ::c_int = 0x00020000;
 
+pub const SS_ONSTACK: ::c_int = 1;
+pub const SS_DISABLE: ::c_int = 2;
+
 pub const FIONBIO: ::c_int = 0x8004667e;
 
 pub const SIGCHLD: ::c_int = 18;
@@ -762,5 +765,6 @@ extern {
                       buf: *mut ::c_char,
                       buflen: ::size_t) -> *const passwd;
     pub fn readdir(dirp: *mut ::DIR) -> *const ::dirent;
+    pub fn fdatasync(fd: ::c_int) -> ::c_int;
 }
 
