@@ -35,7 +35,7 @@
 //! extern crate regex;
 //! ```
 //!
-//! # First example: find a date
+//! # Example: find a date
 //!
 //! General use of regular expressions in this package involves compiling an
 //! expression and then using it to search, split or replace text. For example,
@@ -59,6 +59,40 @@
 //! not process any escape sequences. For example, `"\\d"` is the same
 //! expression as `r"\d"`.
 //!
+//! # Example: Avoid compiling the same regex in a loop
+//!
+//! It is an anti-pattern to compile the same regular expression in a loop
+//! since compilation is typically expensive. (It takes anywhere from a few
+//! microseconds to a few **milliseconds** depending on the size of the
+//! regex.) Not only is compilation itself expensive, but this also prevents
+//! optimizations that reuse allocations internally to the matching engines.
+//!
+//! In Rust, it can sometimes be a pain to pass regular expressions around if
+//! they're used from inside a helper function. Instead, we recommend using the
+//! [`lazy_static`](https://crates.io/crates/lazy_static) crate to ensure that
+//! regular expressions are compiled exactly once.
+//!
+//! For example:
+//!
+//! ```rust
+//! #[macro_use] extern crate lazy_static;
+//! extern crate regex;
+//!
+//! use regex::Regex;
+//!
+//! fn some_helper_function(text: &str) -> bool {
+//!     lazy_static! {
+//!         static ref RE: Regex = Regex::new("...").unwrap();
+//!     }
+//!     RE.is_match(text)
+//! }
+//!
+//! fn main() {}
+//! ```
+//!
+//! Specifically, in this example, the regex will be compiled when it is used for
+//! the first time. On subsequent uses, it will reuse the previous compilation.
+//!
 //! # The `regex!` macro
 //!
 //! Rust's compile-time meta-programming facilities provide a way to write a
@@ -69,15 +103,15 @@
 //! given expression to native Rust code, which ideally makes it faster.
 //! Unfortunately (or fortunately), the dynamic implementation has had a lot
 //! more optimization work put into it currently, so it is faster than
-//! the `regex!` macro in most cases.
+//! the `regex!` macro in almost every case.
 //!
 //! To use the `regex!` macro, you must add `regex_macros` to your dependencies
 //! in your project's `Cargo.toml`:
 //!
 //! ```toml
 //! [dependencies]
-//! regex = "0.1.8"
-//! regex_macros = "0.1.8"
+//! regex = "0.1"
+//! regex_macros = "0.1"
 //! ```
 //!
 //! and then enable the `plugin` feature and import the `regex_macros` crate as
@@ -107,7 +141,7 @@
 //!
 //! **NOTE**: This is implemented using a compiler plugin, which is not
 //! available on the Rust 1.0 beta/stable channels. Therefore, you'll only
-//! be able to use `regex!` on the nightlies.
+//! be able to use `regex!` on the nightly Rust releases.
 //!
 //! # Example: iterating over capture groups
 //!
@@ -176,6 +210,34 @@
 //! # }
 //! ```
 //!
+//! # Example: match multiple regular expressions simultaneously
+//!
+//! This demonstrates how to use a `RegexSet` to match multiple (possibly
+//! overlapping) regular expressions in a single scan of the search text:
+//!
+//! ```rust
+//! use regex::RegexSet;
+//!
+//! let set = RegexSet::new(&[
+//!     r"\w+",
+//!     r"\d+",
+//!     r"\pL+",
+//!     r"foo",
+//!     r"bar",
+//!     r"barfoo",
+//!     r"foobar",
+//! ]).unwrap();
+//!
+//! // Iterate over and collect all of the matches.
+//! let matches: Vec<_> = set.matches("foobar").into_iter().collect();
+//! assert_eq!(matches, vec![0, 2, 3, 4, 6]);
+//!
+//! // You can also test whether a particular regex matched:
+//! let matches = set.matches("foobar");
+//! assert!(!matches.matched(5));
+//! assert!(matches.matched(6));
+//! ```
+//!
 //! # Pay for what you use
 //!
 //! With respect to searching text with a regular expression, there are three
@@ -196,18 +258,17 @@
 //!
 //! # Unicode
 //!
-//! This implementation executes regular expressions **only** on sequences of
-//! Unicode scalar values while exposing match locations as byte indices into
-//! the search string.
+//! This implementation executes regular expressions **only** on valid UTF-8
+//! while exposing match locations as byte indices into the search string.
 //!
 //! Currently, only simple case folding is supported. Namely, when matching
 //! case-insensitively, the characters are first mapped using the
 //! [simple case folding](ftp://ftp.unicode.org/Public/UNIDATA/CaseFolding.txt)
 //! mapping.
 //!
-//! Regular expressions themselves are also **only** interpreted as a sequence
-//! of Unicode scalar values. This means you can use Unicode characters
-//! directly in your expression:
+//! Regular expressions themselves are **only** interpreted as a sequence of
+//! Unicode scalar values. This means you can use Unicode characters directly
+//! in your expression:
 //!
 //! ```rust
 //! # extern crate regex; use regex::Regex;
@@ -229,14 +290,24 @@
 //! # }
 //! ```
 //!
+//! # Opt out of Unicode support
+//!
+//! The `bytes` sub-module provides a `Regex` type that can be used to match
+//! on `&[u8]`. By default, text is interpreted as ASCII compatible text with
+//! all Unicode support disabled (e.g., `.` matches any byte instead of any
+//! Unicode codepoint). Unicode support can be selectively enabled with the
+//! `u` flag. See the `bytes` module documentation for more details.
+//!
+//! Note that Unicode support *cannot* be selectively disabled on the main
+//! `Regex` type that matches on `&str`.
+//!
 //! # Syntax
 //!
 //! The syntax supported in this crate is almost in an exact correspondence
 //! with the syntax supported by RE2. It is documented below.
 //!
 //! Note that the regular expression parser and abstract syntax are exposed in
-//! a separate crate,
-//! [`regex-syntax`](../regex_syntax/index.html).
+//! a separate crate, [`regex-syntax`](../regex_syntax/index.html).
 //!
 //! ## Matching one character
 //!
@@ -397,6 +468,14 @@
 //! text`), which means there's no way to cause exponential blow-up like with
 //! some other regular expression engines. (We pay for this by disallowing
 //! features like arbitrary look-ahead and backreferences.)
+//!
+//! When a DFA is used, pathological cases with exponential state blow up are
+//! avoided by constructing the DFA lazily or in an "online" manner. Therefore,
+//! at most one new state can be created for each byte of input. This satisfies
+//! our time complexity guarantees, but can lead to unbounded memory growth
+//! proportional to the size of the input. As a stopgap, the DFA is only
+//! allowed to store a fixed number of states. (When the limit is reached, its
+//! states are wiped and continues on, possibly duplicating previous work.)
 
 #![deny(missing_docs)]
 #![cfg_attr(test, deny(warnings))]
@@ -407,34 +486,134 @@
 
 extern crate aho_corasick;
 extern crate memchr;
+#[cfg(test)] extern crate quickcheck;
 extern crate regex_syntax as syntax;
+extern crate utf8_ranges;
 
-pub use re::{
-    Regex, Error, Captures, SubCaptures, SubCapturesPos, SubCapturesNamed,
+pub use error::Error;
+pub use set::{RegexSet, SetMatches, SetMatchesIntoIter, SetMatchesIter};
+pub use re_unicode::{
+    Regex, Captures, SubCaptures, SubCapturesPos, SubCapturesNamed,
     CaptureNames, FindCaptures, FindMatches,
     Replacer, NoExpand, RegexSplits, RegexSplitsN,
     quote, is_match,
 };
 
+/**
+Match regular expressions on arbitrary bytes.
+
+This module provides a nearly identical API to the one found in the
+top-level of this crate. There are two important differences:
+
+1. Matching is done on `&[u8]` instead of `&str`. Additionally, `Vec<u8>`
+is used where `String` would have been used.
+2. Regular expressions are compiled with Unicode support *disabled* by
+default. This means that while Unicode regular expressions can only match valid
+UTF-8, regular expressions in this module can match arbitrary bytes. Unicode
+support can be selectively enabled via the `u` flag in regular expressions
+provided by this sub-module.
+
+# Example: match null terminated string
+
+This shows how to find all null-terminated strings in a slice of bytes:
+
+```rust
+# use regex::bytes::Regex;
+let re = Regex::new(r"(?P<cstr>[^\x00]+)\x00").unwrap();
+let text = b"foo\x00bar\x00baz\x00";
+
+// Extract all of the strings without the null terminator from each match.
+// The unwrap is OK here since a match requires the `cstr` capture to match.
+let cstrs: Vec<&[u8]> =
+    re.captures_iter(text)
+      .map(|c| c.name("cstr").unwrap())
+      .collect();
+assert_eq!(vec![&b"foo"[..], &b"bar"[..], &b"baz"[..]], cstrs);
+```
+
+# Example: selectively enable Unicode support
+
+This shows how to match an arbitrary byte pattern followed by a UTF-8 encoded
+string (e.g., to extract a title from a Matroska file):
+
+```rust
+# use std::str;
+# use regex::bytes::Regex;
+let re = Regex::new(r"\x7b\xa9(?:[\x80-\xfe]|[\x40-\xff].)(?u:(.*))").unwrap();
+let text = b"\x12\xd0\x3b\x5f\x7b\xa9\x85\xe2\x98\x83\x80\x98\x54\x76\x68\x65";
+let caps = re.captures(text).unwrap();
+
+// Notice that despite the `.*` at the end, it will only match valid UTF-8
+// because Unicode mode was enabled with the `u` flag. Without the `u` flag,
+// the `.*` would match the rest of the bytes.
+assert_eq!((7, 10), caps.pos(1).unwrap());
+
+// If there was a match, Unicode mode guarantees that `title` is valid UTF-8.
+let title = str::from_utf8(caps.at(1).unwrap()).unwrap();
+assert_eq!("☃", title);
+```
+
+In general, if the Unicode flag is enabled in a capture group and that capture
+is part of the overall match, then the capture is *guaranteed* to be valid
+UTF-8.
+
+# Syntax
+
+The supported syntax is pretty much the same as the syntax for Unicode
+regular expressions with a few changes that make sense for matching arbitrary
+bytes:
+
+1. A new flag, `u`, is available for switching to Unicode mode.
+2. By default, `u` is disabled, which roughly corresponds to "ASCII compatible"
+mode.
+3. In ASCII compatible mode, neither Unicode codepoints nor Unicode character
+classes are allowed.
+4. In ASCII compatible mode, Perl character classes (`\w`, `\d` and `\s`)
+revert to their typical ASCII definition. `\w` maps to `[[:word:]]`, `\d` maps
+to `[[:digit:]]` and `\s` maps to `[[:space:]]`.
+5. In ASCII compatible mode, word boundaries use the ASCII compatible `\w` to
+determine whether a byte is a word byte or not.
+6. Hexadecimal notation can be used to specify arbitrary bytes instead of
+Unicode codepoints. For example, in ASCII compatible mode, `\xFF` matches the
+literal byte `\xFF`, while in Unicode mode, `\xFF` is a Unicode codepoint that
+matches its UTF-8 encoding of `\xC3\xBF`. Similarly for octal notation.
+7. `.` matches any *byte* except for `\n` instead of any codepoint. When the
+`s` flag is enabled, `.` matches any byte.
+
+# Performance
+
+In general, one should expect performance on `&[u8]` to be roughly similar to
+performance on `&str`.
+*/
+pub mod bytes {
+    pub use re_bytes::*;
+}
+
 mod backtrack;
-mod char;
+mod utf8;
 mod compile;
+mod dfa;
+mod error;
+mod exec;
+mod expand;
 mod input;
-mod inst;
-mod pool;
-mod prefix;
-mod program;
+mod literals;
 mod nfa;
-mod re;
+mod pool;
+mod prog;
+mod re_bytes;
+mod re_unicode;
+mod set;
+mod sparse;
 
 /// The `internal` module exists to support the `regex!` macro and other
 /// suspicious activity, such as testing different matching engines.
 #[doc(hidden)]
 pub mod internal {
-    pub use char::Char;
-    pub use input::{Input, CharInput, InputAt};
-    pub use inst::{Inst, EmptyLook, InstRanges};
-    pub use program::{Program, MatchEngine};
-    pub use re::ExNative;
-    pub use re::Regex::{Dynamic, Native};
+    pub use compile::Compiler;
+    pub use exec::{Exec, ExecBuilder};
+    pub use input::{Char, Input, CharInput, InputAt};
+    pub use literals::Literals;
+    pub use prog::{Program, Inst, EmptyLook, InstRanges};
+    pub use re_unicode::{_Regex, ExNative};
 }
