@@ -17,6 +17,7 @@ use std::hash;
 use std::error::Error;
 
 use std::result;
+use std::str;
 
 use parser;
 
@@ -29,7 +30,7 @@ pub enum Identifier {
     /// An identifier that's solely numbers.
     Numeric(u64),
     /// An identifier with letters and numbers.
-    AlphaNumeric(String)
+    AlphaNumeric(String),
 }
 
 impl fmt::Display for Identifier {
@@ -64,6 +65,7 @@ pub struct Version {
 /// Currently, just a generic error. Will make this nicer later.
 #[derive(Clone,PartialEq,Debug,PartialOrd)]
 pub enum SemVerError {
+    /// An error ocurred while parsing.
     ParseError(String),
 }
 
@@ -82,7 +84,7 @@ impl Error for SemVerError {
         }
     }
 }
- 
+
 /// A Result type for errors
 pub type Result<T> = result::Result<T, SemVerError>;
 
@@ -136,6 +138,13 @@ impl Version {
     }
 }
 
+impl str::FromStr for Version {
+    type Err = SemVerError;
+
+    fn from_str(s: &str) -> Result<Version> {
+        Version::parse(s)
+    }
+}
 
 impl fmt::Display for Version {
     #[inline]
@@ -144,14 +153,18 @@ impl fmt::Display for Version {
         if !self.pre.is_empty() {
             try!(write!(f, "-"));
             for (i, x) in self.pre.iter().enumerate() {
-                if i != 0 { try!(write!(f, ".")) };
+                if i != 0 {
+                    try!(write!(f, "."))
+                }
                 try!(write!(f, "{}", x));
             }
         }
         if !self.build.is_empty() {
             try!(write!(f, "+"));
             for (i, x) in self.build.iter().enumerate() {
-                if i != 0 { try!(write!(f, ".")) };
+                if i != 0 {
+                    try!(write!(f, "."))
+                }
                 try!(write!(f, "{}", x));
             }
         }
@@ -165,10 +178,8 @@ impl cmp::PartialEq for Version {
         // We should ignore build metadata here, otherwise versions v1 and v2
         // can exist such that !(v1 < v2) && !(v1 > v2) && v1 != v2, which
         // violate strict total ordering rules.
-        self.major == other.major &&
-            self.minor == other.minor &&
-            self.patch == other.patch &&
-            self.pre == other.pre
+        self.major == other.major && self.minor == other.minor && self.patch == other.patch &&
+        self.pre == other.pre
     }
 }
 
@@ -202,7 +213,7 @@ impl cmp::Ord for Version {
             (0, 0) => Ordering::Equal,
             (0, _) => Ordering::Greater,
             (_, 0) => Ordering::Less,
-            (_, _) => self.pre.cmp(&other.pre)
+            (_, _) => self.pre.cmp(&other.pre),
         }
     }
 }
@@ -226,95 +237,107 @@ mod tests {
     #[test]
     fn test_parse() {
         fn parse_error(e: &str) -> result::Result<Version, SemVerError> {
-            return Err(SemVerError::ParseError(e.to_string()))
+            return Err(SemVerError::ParseError(e.to_string()));
         }
 
-        assert_eq!(Version::parse(""),          parse_error("Parse error"));
-        assert_eq!(Version::parse("  "),        parse_error("Parse error"));
-        assert_eq!(Version::parse("1"),         parse_error("Parse error"));
-        assert_eq!(Version::parse("1.2"),       parse_error("Parse error"));
-        assert_eq!(Version::parse("1.2.3-"),    parse_error("Failed with unparsed input: '-'"));
-        assert_eq!(Version::parse("a.b.c"),     parse_error("Parse error"));
-        assert_eq!(Version::parse("1.2.3 abc"), parse_error("Failed with unparsed input: ' abc'"));
+        assert_eq!(Version::parse(""), parse_error("Parse error"));
+        assert_eq!(Version::parse("  "), parse_error("Parse error"));
+        assert_eq!(Version::parse("1"), parse_error("Parse error"));
+        assert_eq!(Version::parse("1.2"), parse_error("Parse error"));
+        assert_eq!(Version::parse("1.2.3-"),
+                   parse_error("Failed with unparsed input: '-'"));
+        assert_eq!(Version::parse("a.b.c"), parse_error("Parse error"));
+        assert_eq!(Version::parse("1.2.3 abc"),
+                   parse_error("Failed with unparsed input: ' abc'"));
 
-        assert_eq!(Version::parse("1.2.3"), Ok(Version {
-            major: 1,
-            minor: 2,
-            patch: 3,
-            pre: Vec::new(),
-            build: Vec::new(),
-        }));
-        assert_eq!(Version::parse("  1.2.3  "), Ok(Version {
-            major: 1,
-            minor: 2,
-            patch: 3,
-            pre: Vec::new(),
-            build: Vec::new(),
-        }));
-        assert_eq!(Version::parse("1.2.3-alpha1"), Ok(Version {
-            major: 1,
-            minor: 2,
-            patch: 3,
-            pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
-            build: Vec::new(),
-        }));
-        assert_eq!(Version::parse("  1.2.3-alpha1  "), Ok(Version {
-            major: 1,
-            minor: 2,
-            patch: 3,
-            pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
-            build: Vec::new(),
-        }));
-        assert_eq!(Version::parse("1.2.3+build5"), Ok(Version {
-            major: 1,
-            minor: 2,
-            patch: 3,
-            pre: Vec::new(),
-            build: vec![Identifier::AlphaNumeric(String::from("build5"))],
-        }));
-        assert_eq!(Version::parse("  1.2.3+build5  "), Ok(Version {
-            major: 1,
-            minor: 2,
-            patch: 3,
-            pre: Vec::new(),
-            build: vec![Identifier::AlphaNumeric(String::from("build5"))],
-        }));
-        assert_eq!(Version::parse("1.2.3-alpha1+build5"), Ok(Version {
-            major: 1,
-            minor: 2,
-            patch: 3,
-            pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
-            build: vec![Identifier::AlphaNumeric(String::from("build5"))],
-        }));
-        assert_eq!(Version::parse("  1.2.3-alpha1+build5  "), Ok(Version {
-            major: 1,
-            minor: 2,
-            patch: 3,
-            pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
-            build: vec![Identifier::AlphaNumeric(String::from("build5"))],
-        }));
-        assert_eq!(Version::parse("1.2.3-1.alpha1.9+build5.7.3aedf  "), Ok(Version {
-            major: 1,
-            minor: 2,
-            patch: 3,
-            pre: vec![Identifier::Numeric(1),
+        assert_eq!(Version::parse("1.2.3"),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: Vec::new(),
+                       build: Vec::new(),
+                   }));
+        assert_eq!(Version::parse("  1.2.3  "),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: Vec::new(),
+                       build: Vec::new(),
+                   }));
+        assert_eq!(Version::parse("1.2.3-alpha1"),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
+                       build: Vec::new(),
+                   }));
+        assert_eq!(Version::parse("  1.2.3-alpha1  "),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
+                       build: Vec::new(),
+                   }));
+        assert_eq!(Version::parse("1.2.3+build5"),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: Vec::new(),
+                       build: vec![Identifier::AlphaNumeric(String::from("build5"))],
+                   }));
+        assert_eq!(Version::parse("  1.2.3+build5  "),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: Vec::new(),
+                       build: vec![Identifier::AlphaNumeric(String::from("build5"))],
+                   }));
+        assert_eq!(Version::parse("1.2.3-alpha1+build5"),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
+                       build: vec![Identifier::AlphaNumeric(String::from("build5"))],
+                   }));
+        assert_eq!(Version::parse("  1.2.3-alpha1+build5  "),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
+                       build: vec![Identifier::AlphaNumeric(String::from("build5"))],
+                   }));
+        assert_eq!(Version::parse("1.2.3-1.alpha1.9+build5.7.3aedf  "),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: vec![Identifier::Numeric(1),
                       Identifier::AlphaNumeric(String::from("alpha1")),
                       Identifier::Numeric(9),
             ],
-            build: vec![Identifier::AlphaNumeric(String::from("build5")),
+                       build: vec![Identifier::AlphaNumeric(String::from("build5")),
                         Identifier::Numeric(7),
                         Identifier::AlphaNumeric(String::from("3aedf")),
             ],
-        }));
-        assert_eq!(Version::parse("0.4.0-beta.1+0851523"), Ok(Version {
-            major: 0,
-            minor: 4,
-            patch: 0,
-            pre: vec![Identifier::AlphaNumeric(String::from("beta")),
+                   }));
+        assert_eq!(Version::parse("0.4.0-beta.1+0851523"),
+                   Ok(Version {
+                       major: 0,
+                       minor: 4,
+                       patch: 0,
+                       pre: vec![Identifier::AlphaNumeric(String::from("beta")),
                       Identifier::Numeric(1),
             ],
-            build: vec![Identifier::AlphaNumeric(String::from("0851523"))],
-        }));
+                       build: vec![Identifier::AlphaNumeric(String::from("0851523"))],
+                   }));
 
     }
 
@@ -377,17 +400,20 @@ mod tests {
     #[test]
     fn test_eq() {
         assert_eq!(Version::parse("1.2.3"), Version::parse("1.2.3"));
-        assert_eq!(Version::parse("1.2.3-alpha1"), Version::parse("1.2.3-alpha1"));
-        assert_eq!(Version::parse("1.2.3+build.42"), Version::parse("1.2.3+build.42"));
-        assert_eq!(Version::parse("1.2.3-alpha1+42"), Version::parse("1.2.3-alpha1+42"));
+        assert_eq!(Version::parse("1.2.3-alpha1"),
+                   Version::parse("1.2.3-alpha1"));
+        assert_eq!(Version::parse("1.2.3+build.42"),
+                   Version::parse("1.2.3+build.42"));
+        assert_eq!(Version::parse("1.2.3-alpha1+42"),
+                   Version::parse("1.2.3-alpha1+42"));
         assert_eq!(Version::parse("1.2.3+23"), Version::parse("1.2.3+42"));
     }
 
     #[test]
     fn test_ne() {
-        assert!(Version::parse("0.0.0")       != Version::parse("0.0.1"));
-        assert!(Version::parse("0.0.0")       != Version::parse("0.1.0"));
-        assert!(Version::parse("0.0.0")       != Version::parse("1.0.0"));
+        assert!(Version::parse("0.0.0") != Version::parse("0.0.1"));
+        assert!(Version::parse("0.0.0") != Version::parse("0.1.0"));
+        assert!(Version::parse("0.0.0") != Version::parse("1.0.0"));
         assert!(Version::parse("1.2.3-alpha") != Version::parse("1.2.3-beta"));
     }
 
@@ -405,42 +431,46 @@ mod tests {
 
     #[test]
     fn test_to_string() {
-        assert_eq!(Version::parse("1.2.3").unwrap().to_string(), "1.2.3".to_string());
-        assert_eq!(Version::parse("1.2.3-alpha1").unwrap().to_string(), "1.2.3-alpha1".to_string());
-        assert_eq!(Version::parse("1.2.3+build.42").unwrap().to_string(), "1.2.3+build.42".to_string());
-        assert_eq!(Version::parse("1.2.3-alpha1+42").unwrap().to_string(), "1.2.3-alpha1+42".to_string());
+        assert_eq!(Version::parse("1.2.3").unwrap().to_string(),
+                   "1.2.3".to_string());
+        assert_eq!(Version::parse("1.2.3-alpha1").unwrap().to_string(),
+                   "1.2.3-alpha1".to_string());
+        assert_eq!(Version::parse("1.2.3+build.42").unwrap().to_string(),
+                   "1.2.3+build.42".to_string());
+        assert_eq!(Version::parse("1.2.3-alpha1+42").unwrap().to_string(),
+                   "1.2.3-alpha1+42".to_string());
     }
 
     #[test]
     fn test_lt() {
-        assert!(Version::parse("0.0.0")          < Version::parse("1.2.3-alpha2"));
-        assert!(Version::parse("1.0.0")          < Version::parse("1.2.3-alpha2"));
-        assert!(Version::parse("1.2.0")          < Version::parse("1.2.3-alpha2"));
-        assert!(Version::parse("1.2.3-alpha1")   < Version::parse("1.2.3"));
-        assert!(Version::parse("1.2.3-alpha1")   < Version::parse("1.2.3-alpha2"));
+        assert!(Version::parse("0.0.0") < Version::parse("1.2.3-alpha2"));
+        assert!(Version::parse("1.0.0") < Version::parse("1.2.3-alpha2"));
+        assert!(Version::parse("1.2.0") < Version::parse("1.2.3-alpha2"));
+        assert!(Version::parse("1.2.3-alpha1") < Version::parse("1.2.3"));
+        assert!(Version::parse("1.2.3-alpha1") < Version::parse("1.2.3-alpha2"));
         assert!(!(Version::parse("1.2.3-alpha2") < Version::parse("1.2.3-alpha2")));
-        assert!(!(Version::parse("1.2.3+23")     < Version::parse("1.2.3+42")));
+        assert!(!(Version::parse("1.2.3+23") < Version::parse("1.2.3+42")));
     }
 
     #[test]
     fn test_le() {
-        assert!(Version::parse("0.0.0")        <= Version::parse("1.2.3-alpha2"));
-        assert!(Version::parse("1.0.0")        <= Version::parse("1.2.3-alpha2"));
-        assert!(Version::parse("1.2.0")        <= Version::parse("1.2.3-alpha2"));
+        assert!(Version::parse("0.0.0") <= Version::parse("1.2.3-alpha2"));
+        assert!(Version::parse("1.0.0") <= Version::parse("1.2.3-alpha2"));
+        assert!(Version::parse("1.2.0") <= Version::parse("1.2.3-alpha2"));
         assert!(Version::parse("1.2.3-alpha1") <= Version::parse("1.2.3-alpha2"));
         assert!(Version::parse("1.2.3-alpha2") <= Version::parse("1.2.3-alpha2"));
-        assert!(Version::parse("1.2.3+23")     <= Version::parse("1.2.3+42"));
+        assert!(Version::parse("1.2.3+23") <= Version::parse("1.2.3+42"));
     }
 
     #[test]
     fn test_gt() {
-        assert!(Version::parse("1.2.3-alpha2")   > Version::parse("0.0.0"));
-        assert!(Version::parse("1.2.3-alpha2")   > Version::parse("1.0.0"));
-        assert!(Version::parse("1.2.3-alpha2")   > Version::parse("1.2.0"));
-        assert!(Version::parse("1.2.3-alpha2")   > Version::parse("1.2.3-alpha1"));
-        assert!(Version::parse("1.2.3")          > Version::parse("1.2.3-alpha2"));
+        assert!(Version::parse("1.2.3-alpha2") > Version::parse("0.0.0"));
+        assert!(Version::parse("1.2.3-alpha2") > Version::parse("1.0.0"));
+        assert!(Version::parse("1.2.3-alpha2") > Version::parse("1.2.0"));
+        assert!(Version::parse("1.2.3-alpha2") > Version::parse("1.2.3-alpha1"));
+        assert!(Version::parse("1.2.3") > Version::parse("1.2.3-alpha2"));
         assert!(!(Version::parse("1.2.3-alpha2") > Version::parse("1.2.3-alpha2")));
-        assert!(!(Version::parse("1.2.3+23")     > Version::parse("1.2.3+42")));
+        assert!(!(Version::parse("1.2.3+23") > Version::parse("1.2.3+42")));
     }
 
     #[test]
@@ -450,7 +480,7 @@ mod tests {
         assert!(Version::parse("1.2.3-alpha2") >= Version::parse("1.2.0"));
         assert!(Version::parse("1.2.3-alpha2") >= Version::parse("1.2.3-alpha1"));
         assert!(Version::parse("1.2.3-alpha2") >= Version::parse("1.2.3-alpha2"));
-        assert!(Version::parse("1.2.3+23")     >= Version::parse("1.2.3+42"));
+        assert!(Version::parse("1.2.3+23") >= Version::parse("1.2.3+42"));
     }
 
     #[test]
@@ -473,10 +503,120 @@ mod tests {
                   "1.0.0"];
         let mut i = 1;
         while i < vs.len() {
-            let a = Version::parse(vs[i-1]);
+            let a = Version::parse(vs[i - 1]);
             let b = Version::parse(vs[i]);
             assert!(a < b, "nope {:?} < {:?}", a, b);
             i += 1;
         }
     }
+
+    #[test]
+    fn test_from_str() {
+        assert_eq!("1.2.3".parse(),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: Vec::new(),
+                       build: Vec::new(),
+                   }));
+        assert_eq!("  1.2.3  ".parse(),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: Vec::new(),
+                       build: Vec::new(),
+                   }));
+        assert_eq!("1.2.3-alpha1".parse(),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
+                       build: Vec::new(),
+                   }));
+        assert_eq!("  1.2.3-alpha1  ".parse(),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
+                       build: Vec::new(),
+                   }));
+        assert_eq!("1.2.3+build5".parse(),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: Vec::new(),
+                       build: vec![Identifier::AlphaNumeric(String::from("build5"))],
+                   }));
+        assert_eq!("  1.2.3+build5  ".parse(),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: Vec::new(),
+                       build: vec![Identifier::AlphaNumeric(String::from("build5"))],
+                   }));
+        assert_eq!("1.2.3-alpha1+build5".parse(),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
+                       build: vec![Identifier::AlphaNumeric(String::from("build5"))],
+                   }));
+        assert_eq!("  1.2.3-alpha1+build5  ".parse(),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: vec![Identifier::AlphaNumeric(String::from("alpha1"))],
+                       build: vec![Identifier::AlphaNumeric(String::from("build5"))],
+                   }));
+        assert_eq!("1.2.3-1.alpha1.9+build5.7.3aedf  ".parse(),
+                   Ok(Version {
+                       major: 1,
+                       minor: 2,
+                       patch: 3,
+                       pre: vec![Identifier::Numeric(1),
+                      Identifier::AlphaNumeric(String::from("alpha1")),
+                      Identifier::Numeric(9),
+            ],
+                       build: vec![Identifier::AlphaNumeric(String::from("build5")),
+                        Identifier::Numeric(7),
+                        Identifier::AlphaNumeric(String::from("3aedf")),
+            ],
+                   }));
+        assert_eq!("0.4.0-beta.1+0851523".parse(),
+                   Ok(Version {
+                       major: 0,
+                       minor: 4,
+                       patch: 0,
+                       pre: vec![Identifier::AlphaNumeric(String::from("beta")),
+                      Identifier::Numeric(1),
+            ],
+                       build: vec![Identifier::AlphaNumeric(String::from("0851523"))],
+                   }));
+
+    }
+
+    #[test]
+    fn test_from_str_errors() {
+        fn parse_error(e: &str) -> result::Result<Version, SemVerError> {
+            return Err(SemVerError::ParseError(e.to_string()));
+        }
+
+        assert_eq!("".parse(), parse_error("Parse error"));
+        assert_eq!("  ".parse(), parse_error("Parse error"));
+        assert_eq!("1".parse(), parse_error("Parse error"));
+        assert_eq!("1.2".parse(), parse_error("Parse error"));
+        assert_eq!("1.2.3-".parse(),
+                   parse_error("Failed with unparsed input: '-'"));
+        assert_eq!("a.b.c".parse(), parse_error("Parse error"));
+        assert_eq!("1.2.3 abc".parse(),
+                   parse_error("Failed with unparsed input: ' abc'"));
+    }
 }
