@@ -21,7 +21,7 @@ pub fn get_dbpath_for_term(term: &str) -> Option<PathBuf> {
     let mut dirs_to_search = Vec::new();
     let first_char = match term.chars().next() {
         Some(c) => c,
-        None => return None
+        None => return None,
     };
 
     // Find search directory
@@ -34,13 +34,15 @@ pub fn get_dbpath_for_term(term: &str) -> Option<PathBuf> {
                 dirs_to_search.push(homedir)
             }
             match env::var("TERMINFO_DIRS") {
-                Ok(dirs) => for i in dirs.split(':') {
-                    if i == "" {
-                        dirs_to_search.push(PathBuf::from("/usr/share/terminfo"));
-                    } else {
-                        dirs_to_search.push(PathBuf::from(i));
+                Ok(dirs) => {
+                    for i in dirs.split(':') {
+                        if i == "" {
+                            dirs_to_search.push(PathBuf::from("/usr/share/terminfo"));
+                        } else {
+                            dirs_to_search.push(PathBuf::from(i));
+                        }
                     }
-                },
+                }
                 // Found nothing in TERMINFO_DIRS, use the default paths:
                 // According to  /etc/terminfo/README, after looking at
                 // ~/.terminfo, ncurses will search /etc/terminfo, then
